import os
from django import template
from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.management import call_command
from django.utils.html import escape

register = template.Library()


BUNDLE_CACHE = {}
TEMPLATES = {
    'text/css': '<link href="%(url)s" rel="stylesheet" type="%(mimetype)s %(attrs)s/>',
    'text/javascript': '<script src="%(url)s" type="%(mimetype)s" %(attrs)s></script>',
}


@register.simple_tag
def staticbundle(bundle, mimetype=None, **attrs):
    """
    >>> {% staticbundle 'bundlename.css' %}
    >>> {% staticbundle 'bundlename.css' media='screen' %}
    >>> {% staticbundle 'bundlename' mimetype='text/css' %}
    """
    config = getattr(settings, 'STATIC_BUNDLES', {})

    if settings.DEBUG and bundle in config['packages']:
        outdated = False
        src_list = config['packages'][bundle]['src']
        for src in src_list:
            cached_mtime = BUNDLE_CACHE.get(src, 0)
            current_mtime = os.stat(staticfiles_storage.path(src)).st_mtime
            if current_mtime > cached_mtime:
                outdated = True
                BUNDLE_CACHE[src] = current_mtime

        if outdated:
            call_command('compilestatic', bundle)

        if isinstance(src_list, dict):
            src_list = src_list.values()

    else:
        src_list = [bundle]

    output = []
    for src in src_list:
        url = staticfiles_storage.url(src)

        if url.endswith('.css'):
            mimetype = 'text/css'
        elif url.endswith('.js'):
            mimetype = 'text/javascript'

        output.append(TEMPLATES[mimetype] % dict(
            url=url,
            mimetype=mimetype,
            attrs=' '.join('%s="%s"' % (k, escape(v)) for k, v in attrs.iteritems()),
        ))

    return '\n'.join(output)



import os.path
import shlex
import subprocess
from fnmatch import fnmatch
from optparse import make_option
from django.conf import settings
from django.contrib.staticfiles import finders
from django.core.management.base import BaseCommand
from django.utils.datastructures import SortedDict


def find_static_files(ignore_patterns=()):
    found_files = SortedDict()
    for finder in finders.get_finders():
        for path, storage in finder.list(ignore_patterns):
            found_files[path] = storage.path(path)
    return found_files


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--no-compile', action='store_false', default=True, dest='compile'),
    )

    def get_format_params(self, dst):
        filename = os.path.basename(dst)
        path = os.path.dirname(dst)
        basename, ext = os.path.splitext(filename)
        if path.startswith(settings.STATIC_ROOT):
            relpath = path[len(settings.STATIC_ROOT) + 1:]
        else:
            relpath = path

        return dict(
            name=basename,
            ext=ext,
            filename=filename,
            relpath=relpath,
            abspath=path,
            # TODO: is there a better way to do the relroot?
            relroot=os.sep.join([os.pardir] * (relpath.count(os.sep) + 1),),
            root=os.path.abspath(settings.STATIC_ROOT),
        )

    def parse_command(self, cmd, input, params):
        parsed_cmd = shlex.split(str(cmd).format(input=input, **params))
        # force absolute path to binary
        parsed_cmd[0] = os.path.abspath(parsed_cmd[0])

        # TODO: why is uglify hanging when we pass the command as a list?
        return ' '.join(parsed_cmd)

    def run_command(self, cmd, root, dst, input, params):
        """
        Execute a command, and if successful write it's stdout to ``root``/``dst``.
        """
        use_stdout = '{output}' not in cmd
        if not use_stdout:
            params['output'] = dst
        parsed_cmd = self.parse_command(cmd, input=input, params=params)

        print " ->", parsed_cmd
        proc = subprocess.Popen(
            args=parsed_cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
            cwd=root,
        )
        (stdout, stderr) = proc.communicate()

        assert not proc.returncode, stderr

        if use_stdout:
            # TODO: this should probably change dest to be a temp file
            with open(os.path.join(root, dst), 'w') as fp:
                fp.write(stdout)

    def apply_preprocessors(self, root, src, dst, processors):
        """
        Preprocessors operate based on the source filename, and apply to each
        file individually.
        """
        matches = [(pattern, cmds) for pattern, cmds in processors.iteritems() if fnmatch(src, pattern)]
        if src == dst and not matches:
            return

        params = self.get_format_params(dst)

        src_path = src
        for pattern, cmd_list in matches:
            for cmd in cmd_list:
                self.run_command(cmd, root=root, dst=dst, input=src_path, params=params)
                src_path = dst

    def apply_postcompilers(self, root, src_list, dst, processors):
        """
        Postcompilers operate based on the destination filename. They operate on a collection
        of files, and are expected to take a list of 1+ inputs and generate a single output.
        """
        dst_file = os.path.join(root, dst)

        matches = [(pattern, cmds) for pattern, cmds in processors.iteritems() if fnmatch(dst, pattern)]
        if not matches:
            # We should just concatenate the files
            with open(dst_file, 'w') as dst_fp:
                for src in src_list:
                    with open(os.path.join(root, src)) as src_fp:
                        for chunk in src_fp:
                            dst_fp.write(chunk)
            return

        params = self.get_format_params(dst)

        # TODO: probably doesnt play nice everywhere
        src_names = src_list
        for pattern, cmd_list in processors.iteritems():
            for cmd in cmd_list:
                self.run_command(cmd, root=root, dst=dst, input=' '.join(src_names), params=params)
                src_names = [dst]

    def handle(self, *bundles, **options):
        config = settings.STATIC_BUNDLES
        if not config:
            return

        # First we need to build a mapping of all files using django.contrib.staticfiles
        bundle_mapping = {}
        for bundle_name, bundle_opts in config.get('packages', {}).iteritems():
            if bundles and bundle_name not in bundles:
                continue

            bundle_opts['ext'] = os.path.splitext(bundle_name)[1]
            bundle_opts.setdefault('preprocessors', config.get('preprocessors'))
            bundle_opts.setdefault('postcompilers', config.get('postcompilers'))
            bundle_mapping[bundle_name] = bundle_opts

        for bundle_name, bundle_opts in bundle_mapping.iteritems():
            src_outputs = []
            is_mapping = isinstance(bundle_opts['src'], dict)

            for src_path in bundle_opts['src']:
                if is_mapping:
                    dst_path = bundle_opts['src'][src_path]
                else:
                    dst_path = src_path

                self.apply_preprocessors(
                    settings.STATIC_ROOT,
                    src_path,
                    dst_path,
                    bundle_opts.get('preprocessors'),
                )

                src_outputs.append(dst_path)

            if options['compile']:
                self.apply_postcompilers(
                    settings.STATIC_ROOT,
                    src_outputs,
                    bundle_name,
                    bundle_opts.get('postcompilers'),
                )



#!/usr/bin/env python
"""
django-static-compiler
======================

An extension for Sentry which allows setting hard quotas.

:copyright: (c) 2013 by David Cramer.
:license: Apache License 2.0, see LICENSE for more details.
"""
from setuptools import setup, find_packages


tests_require = [
    'exam',
    'mock',
    'pytest',
    'pytest-django',
    'unittest2',
]

install_requires = [
    'django',
]

setup(
    name='django-static-compiler',
    version='0.1.0',
    author='David Cramer',
    author_email='dcramer@gmail.com',
    url='http://github.com/dcramer/django-static-compiler',
    description='A static file compiler for Django',
    long_description=__doc__,
    license='Apache License 2.0',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    zip_safe=False,
    install_requires=install_requires,
    extras_require={'tests': tests_require},
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)
